<?php
/***********************************************************************

  Copyright (C) 2002, 2003  Rickard Andersson (punbb@telia.com)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


//	This script updates the forum database from version 1.0 Beta 3 to
//  1.0 RC 1. Copy this file to the forum root directory and run it. Then
//  remove it from the root directory or anyone will be able to run it (NOT
//  good!).

$update_from = '1.0 Beta 3';
$update_to = '1.0 RC 1';


@include 'config.php';

// If config.php doesn't exist, PUN shouldn't be defined
if (!defined('PUN'))
	exit('This file must be run from the forum root directory.');


// Turn off PHP time limit
@set_time_limit(0);


function error($message, $file, $line, $db_error = false)
{
	print '<b>An error was encountered</b><br><br>'."\n".'<b>File:</b> '.$file.'<br>'."\n".'<b>Line:</b> '.$line.'<br><br>'."\n".'<b>PunBB reported</b>: '.$message."\n";

	if ($db_error != false)
		print '<br><b>Database reported:</b> '.htmlspecialchars($db_error['error']).' (Errno: '.$db_error['errno'].')'."\n";

	exit();
}


// Update posts, topics, lastpost, lastpostid and lastposter for a forum (orphaned topics are not included)
function update_forum($forum_id)
{
	global $db;

	$result = $db->query('SELECT COUNT(id), SUM(num_replies) FROM '.$db->prefix.'topics WHERE moved_to IS NULL AND forum_id='.$forum_id) or error('Unable to fetch forum topic count', __FILE__, __LINE__, $db->error());
	list($num_topics, $num_posts) = $db->fetch_row($result);

	$num_posts = $num_posts + $num_topics;		// $posts is only the sum of all replies (we have to add the topic posts)

	$result = $db->query('SELECT last_post, last_post_id, last_poster FROM '.$db->prefix.'topics WHERE forum_id='.$forum_id.' AND moved_to IS NULL ORDER BY last_post DESC LIMIT 1') or error('Unable to fetch last_post/last_post_id/last_poster', __FILE__, __LINE__, $db->error());
	if ($db->num_rows($result))		// There are topics in the forum
	{
		list($last_post, $last_post_id, $last_poster) = $db->fetch_row($result);

		$db->query('UPDATE '.$db->prefix.'forums SET num_topics='.$num_topics.', num_posts='.$num_posts.', last_post='.$last_post.', last_post_id='.$last_post_id.', last_poster=\''.addslashes($last_poster).'\' WHERE id='.$forum_id) or error('Unable to update last_post/last_post_id/last_poster', __FILE__, __LINE__, $db->error());
	}
	else	// There are no topics
		$db->query('UPDATE '.$db->prefix.'forums SET num_topics=0, num_posts=0, last_post=NULL, last_post_id=NULL, last_poster=NULL WHERE id='.$forum_id) or error('Unable to update last_post/last_post_id/last_poster', __FILE__, __LINE__, $db->error());
}


// Load DB abstraction layer and try to connect
require 'include/dblayer/commondb.php';


// Check current version
$result = $db->query('SELECT punversion FROM '.$db->prefix.'options');
if (!$result || $db->result($result, 0) != $update_from)
	error('This script can only update version '.$update_from.'. The database "'.$db_name.'" doesn\'t seem to be running that version. Update process aborted.', __FILE__, __LINE__);



// Alerts? We don't need no stinkin' alerts!
$db->query('DROP TABLE '.$db_prefix.'alerts') or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));



// We do, however, need reports :-)
switch ($db_type)
{
	case 'mysql':
		$queries[] = 'CREATE TABLE '.$db_prefix."reports (
					  id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
					  post_id int(10) UNSIGNED NOT NULL DEFAULT '0',
					  topic_id int(10) UNSIGNED NOT NULL DEFAULT '0',
					  forum_id int(10) UNSIGNED NOT NULL DEFAULT '0',
					  reported_by int(10) UNSIGNED NOT NULL DEFAULT '0',
					  created int(10) UNSIGNED NOT NULL DEFAULT '0',
					  message text NOT NULL DEFAULT '',
					  zapped int(10) UNSIGNED DEFAULT NULL,
					  zapped_by int(10) UNSIGNED DEFAULT NULL,
					  PRIMARY KEY (id)
					  ) TYPE=MyISAM;";
		break;

	case 'pgsql':
		$queries[] = 'CREATE TABLE '.$db_prefix."reports (
					  id SERIAL,
					  post_id INT NOT NULL DEFAULT '0',
					  topic_id INT NOT NULL DEFAULT '0',
					  forum_id INT NOT NULL DEFAULT '0',
					  reported_by INT NOT NULL DEFAULT '0',
					  created INT NOT NULL DEFAULT '0',
					  message TEXT NOT NULL DEFAULT '',
					  zapped INT DEFAULT NULL,
					  zapped_by INT DEFAULT NULL,
					  PRIMARY KEY (id)
					  )";
		break;
}

$queries[] = 'CREATE INDEX '.$db_prefix.'reports_zapped_idx ON '.$db_prefix.'reports(zapped)';

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."bans,
					  CHANGE userip ip VARCHAR(15) DEFAULT NULL,
					  CHANGE useremail email VARCHAR(50) DEFAULT NULL";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."bans RENAME userip TO ip";
		$queries[] = 'ALTER TABLE '.$db->prefix."bans RENAME useremail TO email";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."categories,
					  CHANGE catname cat_name VARCHAR(80) NOT NULL DEFAULT 'New Category',
					  CHANGE admmodonly admmod_only TINYINT(1) NOT NULL DEFAULT '0'";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."categories RENAME catname TO cat_name";
		$queries[] = 'ALTER TABLE '.$db->prefix."categories RENAME admmodonly TO admmod_only";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."censoring,
					  CHANGE searchfor search_for VARCHAR(60) NOT NULL DEFAULT '',
					  CHANGE replacewith replace_with VARCHAR(60) NOT NULL DEFAULT ''";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."censoring RENAME searchfor TO search_for";
		$queries[] = 'ALTER TABLE '.$db->prefix."censoring RENAME replacewith TO replace_with";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."forums,
					  CHANGE forumname forum_name VARCHAR(80) NOT NULL DEFAULT 'New forum',
					  CHANGE forumdesc forum_desc TEXT DEFAULT NULL,
					  CHANGE moderators moderators TEXT DEFAULT NULL,
					  CHANGE topics num_topics MEDIUMINT(8) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE posts num_posts MEDIUMINT(8) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE lastpost last_post INT(10) UNSIGNED DEFAULT NULL,
					  CHANGE lastpostid last_post_id INT(10) UNSIGNED DEFAULT NULL,
					  CHANGE lastposter last_poster VARCHAR(25) DEFAULT NULL,
					  CHANGE admmodonly admmod_only TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE catid cat_id INT(10) UNSIGNED NOT NULL DEFAULT '0'";
		$queries[] = 'UPDATE '.$db->prefix."forums SET moderators=NULL";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME forumname TO forum_name";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME forumdesc TO forum_desc";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME topics TO num_topics";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME posts TO num_posts";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME lastpost TO last_post";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME lastpostid TO last_post_id";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME lastposter TO last_poster";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME admmodonly TO admmod_only";
		$queries[] = 'ALTER TABLE '.$db->prefix."forums RENAME catid TO cat_id";
		$queries[] = 'UPDATE '.$db->prefix."forums SET moderators=NULL";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."online,
					  CHANGE userid user_id INT(10) UNSIGNED NOT NULL DEFAULT '0'";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."online RENAME userid TO user_id";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."options
					  CHANGE punversion cur_version VARCHAR(30) NOT NULL DEFAULT '',
					  CHANGE title board_title TEXT NOT NULL DEFAULT '',
					  CHANGE description board_desc TEXT DEFAULT NULL,
					  CHANGE baseurl base_url VARCHAR(100) NOT NULL DEFAULT '',
					  CHANGE adminemail admin_email VARCHAR(50) NOT NULL DEFAULT '',
					  CHANGE mailinglist mailing_list TEXT DEFAULT NULL,
					  CHANGE servertimezone server_timezone TINYINT(2) NOT NULL DEFAULT '0',
					  CHANGE timeformat time_format VARCHAR(15) NOT NULL DEFAULT 'H:i:s',
					  CHANGE dateformat date_format VARCHAR(15) NOT NULL DEFAULT 'Y-m-d',
					  CHANGE topicreview topic_review TINYINT(3) NOT NULL DEFAULT '15',
					  CHANGE disptopicsdefault disp_topics_default TINYINT(3) NOT NULL DEFAULT '30',
					  CHANGE disppostsdefault disp_posts_default TINYINT(3) NOT NULL DEFAULT '25',
					  CHANGE redirectwait redirect_delay TINYINT(3) NOT NULL DEFAULT '1',
					  CHANGE floodinterval flood_interval SMALLINT(5) UNSIGNED NOT NULL DEFAULT '30',
					  CHANGE avatardir avatars_dir VARCHAR(50) NOT NULL DEFAULT 'img/avatars',
					  CHANGE avatarwidth avatars_width SMALLINT(5) UNSIGNED NOT NULL DEFAULT '60',
					  CHANGE avatarheight avatars_height SMALLINT(5) UNSIGNED NOT NULL DEFAULT '60',
					  CHANGE avatarsize avatars_size SMALLINT(5) UNSIGNED NOT NULL DEFAULT '10240',
					  CHANGE searchallforums search_all_forums TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE sigsmilies smilies_sig TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE makelinks make_links TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE showpostcount show_post_count TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE titles ranks TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE defaultstyle default_style VARCHAR(25) NOT NULL DEFAULT 'Light_and_Blue',
					  CHANGE usersonline users_online TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE userpostedearlier show_dot TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE newregs regs_allow TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE validateregs regs_validate TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE smtphost smtp_host VARCHAR(100) DEFAULT NULL,
					  CHANGE smtpuser smtp_user VARCHAR(25) DEFAULT NULL,
					  CHANGE smtppass smtp_pass VARCHAR(25) DEFAULT NULL,
					  CHANGE cookietimeout timeout_cookie SMALLINT(6) NOT NULL DEFAULT '600',
					  CHANGE onlinetimeout timeout_online SMALLINT(6) NOT NULL DEFAULT '300',
					  CHANGE indentsize indent_num_spaces TINYINT(3) NOT NULL DEFAULT '4',
					  CHANGE maintenancemess maintenance_message TEXT DEFAULT NULL,
					  CHANGE rules rules_message TEXT DEFAULT NULL,
					  CHANGE userules rules TINYINT(1) NOT NULL DEFAULT '0',
					  ADD report_method TINYINT(1) NOT NULL DEFAULT '0',
					  ADD webmaster_email VARCHAR(50) NOT NULL DEFAULT ''";
		$queries[] = 'UPDATE '.$db->prefix."options SET webmaster_email=admin_email";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME punversion TO cur_version";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME title TO board_title";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME description TO board_desc";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME baseurl TO base_url";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME adminemail TO admin_email";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME mailinglist TO mailing_list";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME servertimezone TO server_timezone";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME timeformat TO time_format";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME dateformat TO date_format";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME topicreview TO topic_review";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME disptopicsdefault TO disp_topics_default";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME disppostsdefault TO disp_posts_default";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME redirectwait TO redirect_delay";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME floodinterval TO flood_interval";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME avatardir TO avatars_dir";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME avatarwidth TO avatars_width";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME avatarheight TO avatars_height";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME avatarsize TO avatars_size";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME searchallforums TO search_all_forums";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME sigsmilies TO smilies_sig";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME makelinks TO make_links";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME showpostcount TO show_post_count";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME titles TO ranks";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME defaultstyle TO default_style";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME usersonline TO users_online";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME userpostedearlier TO show_dot";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME newregs TO regs_allow";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME validateregs TO regs_validate";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME smtphost TO smtp_host";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME smtpuser TO smtp_user";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME smtppass TO smtp_pass";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME cookietimeout TO timeout_cookie";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME onlinetimeout TO timeout_online";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME indentsize TO indent_num_spaces";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME maintenancemess TO maintenance_message";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME rules TO rules_message";
		$queries[] = 'ALTER TABLE '.$db->prefix."options RENAME userules TO rules";
		$queries[] = 'ALTER TABLE '.$db->prefix."options ADD report_method SMALLINT";
		$queries[] = 'ALTER TABLE '.$db->prefix."options ALTER report_method SET DEFAULT '0'";
		$queries[] = 'ALTER TABLE '.$db->prefix."options ADD webmaster_email VARCHAR(50)";
		$queries[] = 'UPDATE '.$db->prefix."options SET webmaster_email=admin_email";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions,
					  CHANGE html message_html TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE puntags message_bbcode TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE imgtag message_img_tag TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE guestread guests_read TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE guestpost guests_post TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE guestposttopic guests_post_topic TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE guestsearch guests_search TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE userpost users_post TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE userposttopic users_post_topic TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE usereditpost users_edit_post TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE userdelpost users_del_post TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE userdeltopic users_del_topic TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE allcapssubject subject_all_caps TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE allcapsmessage message_all_caps TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE allcapssignature sig_all_caps TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE sightml sig_html TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE sigpuntags sig_bbcode TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE sigimgtag sig_img_tag TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE siglength sig_length SMALLINT(5) NOT NULL DEFAULT '400',
					  CHANGE siglines sig_lines TINYINT(3) NOT NULL DEFAULT '4',
					  CHANGE allowbanemail allow_banned_email TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE allowdupemail allow_dupe_email TINYINT(1) NOT NULL DEFAULT '0',
					  ADD users_set_title TINYINT(1) NOT NULL DEFAULT '0' AFTER users_del_topic";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME html TO message_html";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME puntags TO message_bbcode";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME imgtag TO message_img_tag";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME guestread TO guests_read";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME guestpost TO guests_post";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME guestposttopic TO guests_post_topic";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME guestsearch TO guests_search";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME userpost TO users_post";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME userposttopic TO users_post_topic";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME usereditpost TO users_edit_post";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME userdelpost TO users_del_post";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME userdeltopic TO users_del_topic";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME allcapssubject TO subject_all_caps";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME allcapsmessage TO message_all_caps";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME allcapssignature TO sig_all_caps";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME sightml TO sig_html";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME sigpuntags TO sig_bbcode";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME sigimgtag TO sig_img_tag";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME siglength TO sig_length";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME siglines TO sig_lines";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME allowbanemail TO allow_banned_email";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions RENAME allowdupemail TO allow_dupe_email";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions ADD users_set_title SMALLINT";
		$queries[] = 'ALTER TABLE '.$db->prefix."permissions ALTER users_set_title SET DEFAULT '1'";
		$queries[] = 'UPDATE '.$db->prefix."permissions SET users_set_title=1";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."posts,
					  CHANGE posterid poster_id INT(10) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE posterip poster_ip VARCHAR(15) DEFAULT NULL,
					  CHANGE posteremail poster_email VARCHAR(50) DEFAULT NULL,
					  CHANGE editedby edited_by VARCHAR(25) DEFAULT NULL,
					  CHANGE topicid topic_id INT(10) UNSIGNED NOT NULL DEFAULT '0'";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."posts RENAME posterid TO poster_id";
		$queries[] = 'ALTER TABLE '.$db->prefix."posts RENAME posterip TO poster_ip";
		$queries[] = 'ALTER TABLE '.$db->prefix."posts RENAME posteremail TO poster_email";
		$queries[] = 'ALTER TABLE '.$db->prefix."posts RENAME editedby TO edited_by";
		$queries[] = 'ALTER TABLE '.$db->prefix."posts RENAME topicid TO topic_id";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."titles
					  RENAME ".$db->prefix."ranks,
					  CHANGE title rank VARCHAR(50) NOT NULL DEFAULT '',
					  CHANGE minposts min_posts MEDIUMINT(8) UNSIGNED NOT NULL DEFAULT '0'";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."titles RENAME TO ".$db->prefix."ranks";
		$queries[] = 'ALTER TABLE '.$db->prefix."ranks RENAME title TO rank";
		$queries[] = 'ALTER TABLE '.$db->prefix."ranks RENAME minposts TO min_posts";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."searchmatches,
					  RENAME ".$db->prefix."search_matches,
					  CHANGE postid post_id INT(10) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE wordid word_id INT(10) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE subjectmatch subject_match TINYINT(1) NOT NULL DEFAULT '0'";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."searchmatches RENAME TO ".$db->prefix."search_matches";
		$queries[] = 'ALTER TABLE '.$db->prefix."search_matches RENAME postid TO post_id";
		$queries[] = 'ALTER TABLE '.$db->prefix."search_matches RENAME wordid TO word_id";
		$queries[] = 'ALTER TABLE '.$db->prefix."search_matches RENAME subjectmatch TO subject_match";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."searchresults,
					  RENAME ".$db->prefix."search_results,
					  CHANGE searchdata search_data TEXT NOT NULL";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."searchresults RENAME TO ".$db->prefix."search_results";
		$queries[] = 'ALTER TABLE '.$db->prefix."search_results RENAME searchdata TO search_data";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."searchwords,
					  RENAME ".$db->prefix."search_words";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."searchwords RENAME TO ".$db->prefix."search_words";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
		$queries[] = 'ALTER TABLE '.$db->prefix."topics
					  CHANGE lastpost last_post INT(10) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE lastpostid last_post_id INT(10) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE lastposter last_poster VARCHAR(25) DEFAULT NULL,
					  CHANGE views num_views MEDIUMINT(8) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE replies num_replies MEDIUMINT(8) UNSIGNED NOT NULL DEFAULT '0',
					  CHANGE ghost moved_to INT(10) UNSIGNED DEFAULT NULL,
					  CHANGE forumid forum_id INT(10) UNSIGNED NOT NULL DEFAULT '0'";
		break;

	case 'pgsql':
		$queries[] = 'ALTER TABLE '.$db->prefix."topics RENAME lastpost TO last_post";
		$queries[] = 'ALTER TABLE '.$db->prefix."topics RENAME lastpostid TO last_post_id";
		$queries[] = 'ALTER TABLE '.$db->prefix."topics RENAME lastposter TO last_poster";
		$queries[] = 'ALTER TABLE '.$db->prefix."topics RENAME views TO num_views";
		$queries[] = 'ALTER TABLE '.$db->prefix."topics RENAME replies TO num_replies";
		$queries[] = 'ALTER TABLE '.$db->prefix."topics RENAME ghost TO moved_to";
		$queries[] = 'ALTER TABLE '.$db->prefix."topics RENAME forumid TO forum_id";
		break;
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));
$queries = array();



switch ($db_type)
{
	case 'mysql':
	{
		$queries[] = 'ALTER TABLE '.$db->prefix."users
					  ADD title VARCHAR(50) DEFAULT NULL AFTER email,
					  CHANGE id id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
					  CHANGE useavatar use_avatar TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE disptopics disp_topics TINYINT(3) UNSIGNED DEFAULT NULL,
					  CHANGE dispposts disp_posts TINYINT(3) UNSIGNED DEFAULT NULL,
					  CHANGE hideemail hide_email TINYINT(1) NOT NULL DEFAULT '0',
					  CHANGE savepass save_pass TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE showimages show_img TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE showsignatures show_sig TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE linktonewwin link_to_new_win TINYINT(1) NOT NULL DEFAULT '1',
					  CHANGE posts num_posts INT(10) UNSIGNED DEFAULT '0' NOT NULL,
					  CHANGE lastpost last_post INT(10) UNSIGNED DEFAULT NULL,
					  CHANGE adminnote admin_note VARCHAR(30) DEFAULT NULL,
					  CHANGE newpass activate_string VARCHAR(50) DEFAULT NULL,
					  CHANGE newpasskey activate_key VARCHAR(8) DEFAULT NULL,
					  DROP newemail,
					  DROP newemailkey";

		// MySQL screws up the ID for the row with ID=0 (AUTO_INCREMENT) when we use ALTER TABLE
		$queries[] = 'UPDATE '.$db->prefix."users SET id=0 WHERE username='Guest'";
		break;
	}

	case 'pgsql':
	{
		$queries[] = 'ALTER TABLE '.$db->prefix."users ADD title VARCHAR(50)";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME useavatar TO use_avatar";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME disptopics TO disp_topics";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME dispposts TO disp_posts";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME hideemail TO hide_email";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME savepass TO save_pass";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME showimages TO show_img";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME showsignatures TO show_sig";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME linktonewwin TO link_to_new_win";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME posts TO num_posts";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME lastpost TO last_post";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME adminnote TO admin_note";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME newpass TO activate_string";
		$queries[] = 'ALTER TABLE '.$db->prefix."users RENAME newpasskey TO activate_key";
		// newemail and newemailkey can be dropped manually (it's too much work to do here since postgresql < 7.3 doesn't support DROP COLUMN)
		break;
	}
}

@reset($queries);
while (list(, $query) = @each($queries))
	$db->query($query) or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));


// This feels like a good time to update lastpost/lastposter for all forums
$result = $db->query('SELECT id FROM '.$db->prefix.'forums') or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));

while ($row = $db->fetch_row($result))
	update_forum($row[0]);


// We'll empty the search results table as well (we don't use TRUNCATE since we're in a transaction)
$db->query('DELETE FROM '.$db->prefix.'search_results') or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));


// Update version information in database
$db->query('UPDATE '.$db->prefix.'options SET cur_version=\''.$update_to.'\'') or exit('Error on line: '.__LINE__.'<br>'.$db_type.' reported: '.current($db->error()));


// End transaction (if supported)
switch ($db_type)
{
	case 'pgsql':
		$db->query('COMMIT') or exit('Unable to commit transaction. Please check your settings and try again. <a href="JavaScript: history.go(-1)">Go back</a>.');
		break;
}


exit('Update successful! Your forum database has now been updated to version '.$update_to.'. You must now remove this script from the forum root directory!');
